@@ -4,117 +4,12 @@
 # (unfortunately, bash won't let me add descriptions to cmds)
 # Kelly Kaoudis kelly.n.kaoudis at intel.com, Aug. 2015
 
-# Constant to indicate command has no options
-NO_OPTS=""
-readonly NO_OPTS
-
-# Associative array of plugins and associated subcommands
-# Order here is same as PLUGIN_OBJS in Makefile
-typeset -Ar _plugin_subcmds=(
-	[intel]="id-ctrl internal-log lat-stats \
-		set-bucket-thresholds lat-stats-tracking \
-		market-name smart-log-add temp-stats"
-	[amzn]="id-ctrl"
-	[memblaze]="smart-log-add get-pm-status set-pm-status \
-		select-download lat-stats lat-stats-print lat-log \
-		lat-log-print clear-error-log"
-	[wdc]="cap-diag drive-log get-crash-dump get-pfail-dump \
-		id-ctrl purge purge-monitor vs-internal-log \
-		vs-nand-stats vs-smart-add-log clear-pcie-correctable-errors \
-		drive-essentials get-drive-status clear-assert-dump \
-		drive-resize vs-fw-activate-history clear-fw-activate-history \
-		enc-get-log vs-telemetry-controller-option \
-		vs-error-reason-identifier log-page-directory \
-		namespace-resize vs-drive-info vs-temperature-stats \
-		capabilities cloud-SSD-plugin-version vs-pcie-stats"
-	[huawei]="list id-ctrl"
-	[netapp]="smdevices ontapdevices"
-	[toshiba]="vs-smart-add-log vs-internal-log \
-		clear-pcie-correctable-errors"
-	[micron]="select-download vs-temperature-stats vs-pcie-stats \
-		clear-pcie-correctable-errors vs-internal-log \
-		vs-telemetry-controller-option vs-nand-stats \
-		vs-drive-info plugin-version cloud-SSD-plugin-version \
-		log-page-directory vs-fw-activate-history \
-		vs-error-reason-identifier vs-smart-add-log \
-		clear-fw-activate-history vs-smbus-option"
-	[seagate]="vs-temperature-stats vs-log-page-sup \
-		vs-smart-add-log vs-pcie-stats clear-pcie-correctable-errors \
-		get-host-tele get-ctrl-tele vs-internal-log \
-		plugin-version"
-	[virtium]="save-smart-to-vtview-log show-identify"
-	[shannon]="smart-log-add get-feature-add set-feature-add id-ctrl"
-	[dera]="smart-log-add"
-	[sfx]="smart-log-add lat-stats get-bad-block query-cap \
-		change-cap set-feature get-feature"
-	[transcend]="healthvalue badblock"
-	[zns]="id-ctrl id-ns zone-mgmt-recv \
-		zone-mgmt-send report-zones close-zone \
-		finish-zone open-zone reset-zone offline-zone \
-		set-zone-desc zone-append changed-zone-list"
-	[nvidia]="id-ctrl"
-	[ymtc]="smart-log-add"
-)
-readonly _plugin_subcmds
-
-# Associative array mapping plugins to coresponding option completions
-typeset -Ar _plugin_funcs=(
-	[intel]="plugin_intel_opts"
-	[amzn]="plugin_amzn_opts"
-	[memblaze]="plugin_memblaze_opts"
-	[wdc]="plugin_wdc_opts"
-	[huawei]="plugin_huawei_opts"
-	[toshiba]="plugin_toshiba_opts"
-	[micron]="plugin_micron_opts"
-	[seagate]="plugin_seagate_opts"
-	[virtium]="plugin_virtium_opts"
-	[shannon]="plugin_shannon_opts"
-	[dera]="plugin_dera_opts"
-	[sfx]="pluginx_sfx_opts"
-	[transcend]="plugin_transcend_opts"
-	[zns]="plugin_zns_opts"
-	[nvidia]="plugin_nvidia_opts"
-	[ymtc]="plugin_ymtc_opts"
-)
-readonly _plugin_funcs
-
-# Top level commands
-_cmds="list list-subsys id-ctrl id-ns \
-	id-ns-granularity list-ns list-ctrl \
-	id-ns-lba-format nvm-id-ns nvm-id-ns-lba-format \
-	nvm-id-ctrl primary-ctrl-caps list-secondary \
-	ns-descs id-nvmset id-uuid id-iocs create-ns \
-	delete-ns get-ns-id get-log telemetry-log \
-	fw-log changed-ns-list-log smart-log ana-log \
-	error-log effects-log endurance-log \
-	predictable-lat-log pred-lat-event-agg-log \
-	persistent-event-log endurance-agg-log \
-	lba-status-log resv-notif-log get-feature \
-	device-self-test self-test-log set-feature \
-	set-property get-property format fw-commit \
-	fw-download admin-passthru io-passthru \
-	security-send security-recv get-lba-status \
-	resv-acquire resv-register resv-release \
-	resv-report dsm copy flush compare read \
-	write write-zeros write-uncor verify \
-	sanitize sanitize-log reset subsystem-reset \
-	ns-rescan show-regs discover connect-all \
-	connect disconnect disconnect-all gen-hostnqn \
-	show-hostnqn dir-receive dir-send virt-mgmt \
-	rpmb boot-part-log fid-support-effects-log \
-	supported-log-pages lockdown media-unit-stat-log \
-	supported-cap-config-log dim"
-
-# Add plugins:
-for plugin in "${!_plugin_subcmds[@]}"; do
-	_cmds+=" $plugin"
-done
-
-cmds+=" version help"
-
 nvme_list_opts () {
-    local opts=""
+	local opts=""
 	local compargs=""
+	local vals=""
+	local opt=""
+	local val=""
 
 	local nonopt_args=0
 	for (( i=0; i < ${#words[@]}-1; i++ )); do
@@ -128,6 +23,19 @@ nvme_list_opts () {
 	fi
 
 	opts+=" "
+	vals+=" "
+
+	if [[ $cur != -* ]] && [[ $cur != "" ]] && [[ $prev == "=" ]] && [[ ${words[$cword-2]} == --* ]]; then
+		opt+="${words[$cword-2]}"
+		val+="$cur"
+	elif [[ $cur == "" ]] && [[ $prev != "=" ]] || [[ $cur == "=" ]] && [[ $prev == --* ]]; then
+		opt+="$prev"
+	elif [[ $cur != "=" ]] && [[ $prev != --* ]] && [[ $prev != "=" ]]; then
+		opt+="$prev"
+		val+="$cur"
+	else
+		opt+="$cur"
+	fi
 
 	# Listed here in the same order as in nvme-builtin.h
 	case "$1" in
@@ -139,11 +47,11 @@ nvme_list_opts () {
 			;;
 		"id-ctrl")
 		opts+=" --raw-binary -b --human-readable -H \
-			--vendor-specific -v --output-format= -o"
+			--vendor-specific -V --output-format= -o"
 			;;
 		"id-ns")
 		opts+=" --namespace-id= -n --raw-binary -b \
-			--human-readable -H --vendor-specific -v \
+			--human-readable -H --vendor-specific -V \
 			--force -f --output-format= -o"
 			;;
 		"id-ns-granularity")
@@ -196,12 +104,17 @@ nvme_list_opts () {
 		opts+=" --endgrp-id= -i --output-format= -o"
 			;;
 		"id-iocs")
-		opts+=" --controller-id= -c --output-format= -o --human-readable -H"
+		opts+=" --controller-id= -c"
+			;;
+		"id-domain")
+		opts+=" --domain-id= -c --output-format= -o"
 			;;
 		"create-ns")
 		opts+=" --nsze= -s --ncap= -c --flbas= -f \
-			--dps= -d --nmic= -n --anagrp-id= -a --nvmset-id= -i \
-			--block-size= -b --timeout= -t--csi= -y"
+			--dps= -d --nmic= -m --anagrp-id= -a --nvmset-id= -i \
+			--block-size= -b --timeout= -t --csi= -y --lbstm= -l \
+			--nphndls= -n --nsze-si= -S --ncap-si= -C --azr -z --rar= -r \
+			--ror= -O --rnumzrwa= -u --phndls= -p --endg-id= -e"
 			;;
 		"delete-ns")
 		opts+=" -namespace-id= -n --timeout= -t"
@@ -217,7 +130,7 @@ nvme_list_opts () {
 			;;
 		"get-log")
 		opts+=" --log-id= -i --log-len= -l --namespace-id= -n \
-			--aen= -a --lpo= -o --lsp= -s --lsi= -S \
+			--aen= -a --lpo= -O --lsp= -s --lsi= -S \
 			--rae -r --uuid-index= -U --csi= -y --ot -O \
 			--raw-binary -b"
 			;;
@@ -225,7 +138,7 @@ nvme_list_opts () {
 		opts+=" --output-format= -o --human-readable -H"
 			;;
 		"telemetry-log")
-		opts+=" --output-file= -o --host-generate= -g \
+		opts+=" --output-file= -O --host-generate= -g \
 			--controller-init -c --data-area= -d"
 			;;
 		"fw-log")
@@ -307,10 +220,10 @@ nvme_list_opts () {
 			--cdw12= -c"
 			;;
 		"set-property")
-		opts+=" --offset= -o --value= -v"
+		opts+=" --offset= -O --value= -V"
 			;;
 		"get-property")
-		opts=+" --offset= -o --human-readable -H"
+		opts=+" --offset= -O --human-readable -H"
 			;;
 		"format")
 		opts+=" --namespace-id= -n --timeout= -t --lbaf= -l \
@@ -320,18 +233,17 @@ nvme_list_opts () {
 		opts+=" --slot= -s --action= -a --bpid= -b"
 			;;
 		"fw-download")
-		opts+=" --fw= -f --xfer= -x --offset= -o"
+		opts+=" --fw= -f --xfer= -x --offset= -O"
 			;;
 		"capacity-mgmt")
-		opts+=" --operation= -f --element-id= -i --cap-lower= -l \
-				--cap-upper= -u"
+		opts+=" --operation= -O --element-id= -i --cap-lower= -l \
+			--cap-upper= -u"
 			;;
 		"lockdown")
-		opts+=" --ofi= -O --ifc= -F --prhbt= -P \
-			-scp= -S --uuid -U"
+		opts+=" --ofi= -O --ifc= -f --prhbt= -p --scp= -s --uuid -U"
 			;;
 		"admin-passthru")
-		opts+=" --opcode= -o --flags= -f --prefil= -p --rsvd= -R \
+		opts+=" --opcode= -O --flags= -f --prefil= -p --rsvd= -R \
 			--namespace-id= -n --data-len= -l --metadata-len= -m \
 			--timeout= -t --cdw2= -2 --cdw3= -3 --cdw10= -4 \
 			--cdw11= -5 --cdw12= -6 --cdw13= -7 --cdw14= -8 \
@@ -340,7 +252,7 @@ nvme_list_opts () {
 			--latency -T"
 			;;
 		"io-passthru")
-		opts+=" --opcode= -o --flags= -f --prefill= -p --rsvd= -R \
+		opts+=" --opcode= -O --flags= -f --prefill= -p --rsvd= -R \
 			--namespace-id= -n --data-len= -l --metadata-len= -m \
 			--timeout= -t --cdw2= -2 --cdw3= -3 --cdw10= -4 \
 			--cdw11= -5 --cdw12= -6 --cdw13= -7 --cdw14= -8 \
@@ -399,7 +311,7 @@ nvme_list_opts () {
 			--metadata= -M --prinfo= -p --app-tag-mask= -m \
 			--app-tag= -a --limited-retry -l \
 			--force-unit-access -f --storage-tag-check -C \
-			--dir-type= -T --dir-spec= -S --dsm= -D --show-command -v \
+			--dir-type= -T --dir-spec= -S --dsm= -D --show-command -V \
 			--dry-run -w --latency -t"
 			;;
 		"read")
@@ -408,7 +320,7 @@ nvme_list_opts () {
 			--metadata= -M --prinfo= -p --app-tag-mask= -m \
 			--app-tag= -a --limited-retry -l \
 			--force-unit-access -f --storage-tag-check -C \
-			--dir-type= -T --dir-spec= -S --dsm= -D --show-command -v \
+			--dir-type= -T --dir-spec= -S --dsm= -D --show-command -V \
 			--dry-run -w --latency -t"
 			;;
 		"write")
@@ -417,19 +329,20 @@ nvme_list_opts () {
 			--metadata= -M --prinfo= -p --app-tag-mask= -m \
 			--app-tag= -a --limited-retry -l \
 			--force-unit-access -f --storage-tag-check -C \
-			--dir-type= -T --dir-spec= -S --dsm= -D --show-command -v \
+			--dir-type= -T --dir-spec= -S --dsm= -D --show-command -V \
 			--dry-run -w --latency -t"
 			;;
-		"write-zeros")
+		"write-zeroes")
 		opts+=" --namespace-id= -n --start-block= -s \
 			--block-count= -c --deac -d --limited-retry -l \
 			--force-unit-access -f --prinfo= -p --ref-tag= -r \
 			--app-tag-mask= -m --app-tag= -a \
-			--storage-tag= -S --storage-tag-check -C"
+			--storage-tag= -S --storage-tag-check -C \
+			--dir-type= -T --dir-spec= -S"
 			;;
 		"write-uncor")
 		opts+=" --namespace-id= -n --start-block= -s \
-			--block-count= -c"
+			--block-count= -c --dir-type= -T --dir-spec= -S"
 			;;
 		"verify")
 		opts+=" --namespace-id= -n --start-block= -s \
@@ -441,6 +354,11 @@ nvme_list_opts () {
 		"sanitize")
 		opts+=" --no-dealloc -d --oipbp -i --owpass= -n \
 			--ause -u --sanact= -a --ovrpat= -p"
+		case $opt in
+			--sanact|-a)
+			vals+=" exit-failure start-block-erase start-overwrite start-crypto-erase"
+				;;
+		esac
 			;;
 		"sanitize-log")
 		opts+=" --rae -r --output-format= -o --human-readable -H \
@@ -524,6 +442,31 @@ nvme_list_opts () {
 			--key= -k --msg= -d --address= -o --blocks= -b \
 			--target= -t"
 			;;
+		"show-topology")
+		opts+=" --output-format= -o --verbose -v --ranking= -r"
+			;;
+		"nvme-mi-recv")
+		opts+=" --opcode= -O --namespace-id= -n --data-len= -l \
+			--nmimt= -m --nmd0= -0 --nmd1= -1 --input-file= -i"
+			;;
+		"nvme-mi-send")
+		opts+=" --opcode= -O --namespace-id= -n --data-len= -l \
+			--nmimt= -m --nmd0= -0 --nmd1= -1 --input-file= -i"
+			;;
+		"get-reg")
+		opts+=" --offset, -O --human-readable -H --cap --vs --cmbloc \
+			--cmbsz --bpinfo --cmbsts --cmbebs --cmbswtp --crto \
+			--pmrcap --pmrsts --pmrebs --pmrswtp --intms --intmc \
+			--cc --csts --nssr --aqa --asq --acq --bprsel --bpmbl \
+			--cmbmsc --nssd --pmrctl --pmrmscl --pmrmscu \
+			--output-format -o --verbose -v"
+			;;
+		"set-reg")
+		opts+=" --offset, -O --value= -V --mmio32 -m --intms= --intmc= \
+			--cc= --csts= --nssr= --aqa= --asq= --acq= --bprsel= \
+			--bpmbl= --cmbmsc= --nssd= --pmrctl= --pmrmscl= \
+			--pmrmscu= --output-format= -o --verbose= -v"
+			;;
 		"version")
 		opts+=$NO_OPTS
 			;;
@@ -532,9 +475,13 @@ nvme_list_opts () {
 			;;
 	esac
 
-	opts+=" -h --help"
+	opts+=" -h --help -j --json"
 
-	COMPREPLY+=( $( compgen $compargs -W "$opts" -- $cur ) )
+	if [[ $vals == " " ]]; then
+		COMPREPLY+=( $( compgen $compargs -W "$opts" -- $cur ) )
+	else
+		COMPREPLY+=( $( compgen $compargs -W "$vals" -- $val ) )
+	fi
 
 	return 0
 }
@@ -1057,7 +1004,7 @@ plugin_shannon_opts () {
 			--data-len= -l --raw-binary -b --cdw11= -c --human-readable -H"
 			;;
 		"set-feature-add")
-		opts+=" --namespace-id= -n --feature-id= -f --value= -v \
+		opts+=" --namespace-id= -n --feature-id= -f --value= -V \
 			--data-len= -l --data= -d --save -s"
 			;;
 		"id-ctrl")
@@ -1154,6 +1101,89 @@ plugin_sfx_opts () {
 	return 0
 }
 
+plugin_solidigm_opts () {
+    local opts=""
+	local compargs=""
+
+	local nonopt_args=0
+	for (( i=0; i < ${#words[@]}-1; i++ )); do
+		if [[ ${words[i]} != -* ]]; then
+			let nonopt_args+=1
+		fi
+	done
+
+	if [ $nonopt_args -eq 3 ]; then
+		opts="/dev/nvme* "
+	fi
+
+	opts+=" "
+
+	case "$1" in
+		"id-ctrl")
+		opts+=" --raw-binary -b --human-readable -H \
+			--vendor-specific -v --output-format= -o \
+			--verbose -v "
+			;;
+		"vs-smart-add-log")
+		opts+="--output-format= -o"
+			;;
+		"garbage-collect-log")
+		opts+="--output-format= -o"
+			;;
+		"vs-internal-log")
+		opts+=" --type= -t --namespace-id= -n \
+		--file-prefix= -p --verbose -v"
+			;;
+		"latency-tracking-log")
+		opts+=" --enable -e --disable -d \
+		--read -r --write -w \
+		--type -t --output-format -o"
+			;;
+		"clear-pcie-correctable-errors")
+		opts+=" --no-uuid -n"
+			;;
+		"parse-telemetry-log")
+		opts+=" --host-generate -g --controller-init -c \
+		--data-area -d --config-file -j \
+		--source-file -s"
+			;;
+		"clear-fw-activate-history")
+		opts+=" --no-uuid -n"
+			;;
+		"vs-fw-activate-history")
+		opts+=" --output-format -o"
+			;;
+		"log-page-directory")
+		opts+=" --output-format -o"
+			;;
+		"vs-drive-info")
+		opts+=" "
+			;;
+		"cloud-SSDplugin-version")
+		opts+=$NO_OPTS
+			;;
+		"market-log")
+		opts+=" --raw-binary -b"
+			;;
+		"smart-log-add")
+		opts+=" --namespace-id= -n --output-format -o"
+			;;
+		"temp-stats")
+		opts+=" --raw-binary -b"
+			;;
+		"version")
+		opts+=$NO_OPTS
+			;;
+		"help")
+		opts+=$NO_OPTS
+			;;
+	esac
+
+	COMPREPLY+=( $( compgen $compargs -W "$opts" -- $cur ) )
+
+	return 0
+}
+
 plugin_transcend_opts () {
     local opts=""
 	local compargs=""
@@ -1330,7 +1360,115 @@ plugin_ymtc_opts () {
 		opts+=" --namespace-id= -n --raw-binary -b"
 			;;
 		"help")
-		opts+=NO_OPTS
+		opts+=$NO_OPTS
+			;;
+	esac
+
+	COMPREPLY+=( $( compgen $compargs -W "$opts" -- $cur ) )
+
+	return 0
+}
+
+plugin_inspur_opts () {
+    local opts=""
+	local compargs=""
+
+	local nonopt_args=0
+	for (( i=0; i < ${#words[@]}-1; i++ )); do
+		if [[ ${words[i]} != -* ]]; then
+			let nonopt_args+=1
+		fi
+	done
+
+	if [ $nonopt_args -eq 3 ]; then
+		opts="/dev/nvme* "
+	fi
+
+	opts+=" "
+
+	case "$1" in
+		"nvme-vendor-log")
+		opts+=$NO_OPTS
+			;;
+		"help")
+		opts+=$NO_OPTS
+			;;
+	esac
+
+	COMPREPLY+=( $( compgen $compargs -W "$opts" -- $cur ) )
+
+	return 0
+}
+
+plugin_ocp_opts () {
+    local opts=""
+	local compargs=""
+
+	local nonopt_args=0
+	for (( i=0; i < ${#words[@]}-1; i++ )); do
+		if [[ ${words[i]} != -* ]]; then
+			let nonopt_args+=1
+		fi
+	done
+
+	if [ $nonopt_args -eq 3 ]; then
+		opts="/dev/nvme* "
+	fi
+
+	opts+=" "
+
+	case "$1" in
+		"smart-add-log")
+		opts+=" --output-format= -o"
+			;;
+		"latency-monitor-log")
+		opts+=" --output-format= -o"
+			;;
+		"set-latency-monitor-feature")
+		opts+=" --active_bucket_timer_threshold= -t \
+			--active_threshold_a= -a --active_threshold_b= -b \
+			--active_threshold_c= -c --active_threshold_d= -d \
+			--active_latency_config= -f \
+			--active_latency_minimum_window= -w \
+			--debug_log_trigger_enable -r --discard_debug_log= -l \
+			--latency_monitor_feature_enable= -e"
+			;;
+		"internal-log")
+		opts+=" --telemetry_type= -t --telemetry_data_area= -a \
+			--output-file= -o"
+			;;
+		"clear-fw-activate-history")
+		opts+=" --no-uuid -n"
+			;;
+		"eol-plp-failure-mode")
+		opts+=" --mode= -m --save -s --sel= -S --no-uuid -n"
+			;;
+		"clear-pcie-correctable-error-counters")
+		opts+=" --no-uuid -n"
+			;;
+		"fw-activate-history")
+		opts+=" --output-format= -o"
+			;;
+		"device-capability-log")
+		opts+=" --output-format= -o"
+			;;
+		"set-dssd-power-state-feature")
+		opts+=" --power-state= -p --no-uuid -n --save -s"
+			;;
+		"telemetry-string-log")
+		opts+=" --output-file= -o"
+			;;
+		"set-telemetry-profile")
+		opts+=" --telemetry-profile-select= -t"
+			;;
+		"set-dssd-async-event-config")
+		opts+=" --enable-panic-notices -e --save -s"
+			;;
+		"get-dssd-power-state-feature")
+		opts+=" --sel= -S"
+			;;
+		"help")
+		opts+=$NO_OPTS
 			;;
 	esac
 
@@ -1343,6 +1481,130 @@ _nvme_subcmds () {
 	local cur prev words cword
 	_init_completion || return
 
+	# Constant to indicate command has no options
+	NO_OPTS=""
+
+	# Associative array of plugins and associated subcommands
+	# Order here is same as PLUGIN_OBJS in Makefile
+	typeset -Ar _plugin_subcmds=(
+		[intel]="id-ctrl internal-log lat-stats \
+			set-bucket-thresholds lat-stats-tracking \
+			market-name smart-log-add temp-stats"
+		[amzn]="id-ctrl"
+		[memblaze]="smart-log-add get-pm-status set-pm-status \
+			select-download lat-stats lat-stats-print lat-log \
+			lat-log-print clear-error-log"
+		[wdc]="cap-diag drive-log get-crash-dump get-pfail-dump \
+			id-ctrl purge purge-monitor vs-internal-log \
+			vs-nand-stats vs-smart-add-log clear-pcie-correctable-errors \
+			drive-essentials get-drive-status clear-assert-dump \
+			drive-resize vs-fw-activate-history clear-fw-activate-history \
+			enc-get-log vs-telemetry-controller-option \
+			vs-error-reason-identifier log-page-directory \
+			namespace-resize vs-drive-info vs-temperature-stats \
+			capabilities cloud-SSD-plugin-version vs-pcie-stats"
+		[huawei]="list id-ctrl"
+		[netapp]="smdevices ontapdevices"
+		[toshiba]="vs-smart-add-log vs-internal-log \
+			clear-pcie-correctable-errors"
+		[micron]="select-download vs-temperature-stats vs-pcie-stats \
+			clear-pcie-correctable-errors vs-internal-log \
+			vs-telemetry-controller-option vs-nand-stats \
+			vs-drive-info plugin-version cloud-SSD-plugin-version \
+			log-page-directory vs-fw-activate-history \
+			vs-error-reason-identifier vs-smart-add-log \
+			clear-fw-activate-history vs-smbus-option"
+		[seagate]="vs-temperature-stats vs-log-page-sup \
+			vs-smart-add-log vs-pcie-stats clear-pcie-correctable-errors \
+			get-host-tele get-ctrl-tele vs-internal-log \
+			plugin-version"
+		[virtium]="save-smart-to-vtview-log show-identify"
+		[shannon]="smart-log-add get-feature-add set-feature-add id-ctrl"
+		[dera]="smart-log-add"
+		[sfx]="smart-log-add lat-stats get-bad-block query-cap \
+			change-cap set-feature get-feature"
+    		[solidigm]="id-ctrl vs-smart-add-log garbage-collect-log \
+			vs-internal-log latency-tracking-log \
+			clear-pcie-correctable-errors parse-telemetry-log \
+			clear-fw-activate-history vs-fw-activate-history log-page-directory \
+			vs-drive-info cloud-SSDplugin-version market-log \
+			smart-log-add temp-stats version help"
+		[transcend]="healthvalue badblock"
+		[zns]="id-ctrl id-ns zone-mgmt-recv \
+			zone-mgmt-send report-zones close-zone \
+			finish-zone open-zone reset-zone offline-zone \
+			set-zone-desc zone-append changed-zone-list"
+		[nvidia]="id-ctrl"
+		[ymtc]="smart-log-add"
+		[inspur]="nvme-vendor-log"
+		[ocp]="smart-add-log latency-monitor-log \
+			set-latency-monitor-feature internal-log \
+			clear-fw-activate-history eol-plp-failure-mode \
+			clear-pcie-correctable-error-counters \
+			vs-fw-activate-history device-capability-log \
+			set-dssd-power-state-feature telemetry-string-log \
+			set-telemetry-profile set-dssd-async-event-config \
+			get-dssd-async-event-config"
+	)
+
+	# Associative array mapping plugins to corresponding option completions
+	typeset -Ar _plugin_funcs=(
+		[intel]="plugin_intel_opts"
+		[amzn]="plugin_amzn_opts"
+		[memblaze]="plugin_memblaze_opts"
+		[wdc]="plugin_wdc_opts"
+		[huawei]="plugin_huawei_opts"
+		[toshiba]="plugin_toshiba_opts"
+		[micron]="plugin_micron_opts"
+		[seagate]="plugin_seagate_opts"
+		[virtium]="plugin_virtium_opts"
+		[shannon]="plugin_shannon_opts"
+		[dera]="plugin_dera_opts"
+		[sfx]="plugin_sfx_opts"
+    		[solidigm]="plugin_solidigm_opts"
+		[transcend]="plugin_transcend_opts"
+		[zns]="plugin_zns_opts"
+		[nvidia]="plugin_nvidia_opts"
+		[ymtc]="plugin_ymtc_opts"
+		[inspur]="plugin_inspur_opts"
+		[ocp]="plugin_ocp_opts"
+	)
+
+	# Top level commands
+	_cmds="list list-subsys id-ctrl id-ns \
+		id-ns-granularity list-ns list-ctrl \
+		id-ns-lba-format nvm-id-ns nvm-id-ns-lba-format \
+		nvm-id-ctrl primary-ctrl-caps list-secondary \
+		ns-descs id-nvmset id-uuid id-iocs id-domain create-ns \
+		delete-ns get-ns-id get-log telemetry-log \
+		fw-log changed-ns-list-log smart-log ana-log \
+		error-log effects-log endurance-log \
+		predictable-lat-log pred-lat-event-agg-log \
+		persistent-event-log endurance-agg-log \
+		lba-status-log resv-notif-log get-feature \
+		device-self-test self-test-log set-feature \
+		set-property get-property format fw-commit \
+		fw-download admin-passthru io-passthru \
+		security-send security-recv get-lba-status \
+		resv-acquire resv-register resv-release \
+		resv-report dsm copy flush compare read \
+		write write-zeros write-uncor verify \
+		sanitize sanitize-log reset subsystem-reset \
+		ns-rescan show-regs discover connect-all \
+		connect disconnect disconnect-all gen-hostnqn \
+		show-hostnqn dir-receive dir-send virt-mgmt \
+		rpmb boot-part-log fid-support-effects-log \
+		supported-log-pages lockdown media-unit-stat-log \
+		supported-cap-config-log dim show-topology list-endgrp \
+		nvme-mi-recv nvme-mi-send get-reg set-reg"
+
+	# Add plugins:
+	for plugin in "${!_plugin_subcmds[@]}"; do
+		_cmds+=" $plugin"
+	done
+
+	_cmds+=" version help"
+
 	if [[ ${#words[*]} -lt 3 ]]; then
 		COMPREPLY+=( $(compgen -W "$_cmds" -- $cur ) )
 	else
